# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# vim:sw=4:ts=4:et

import logging

_LOGGER = logging.getLogger(__name__)
# fmt: off


class Media(object):

    def factory_create(self):
        ret = self.command(
            'mediaFileFind.cgi?action=factory.create'
        )
        return ret.content.decode('utf-8')

    def factory_close(self, factory_id):
        ret = self.command(
            'mediaFileFind.cgi?action=factory.close&object={0}'
            .format(factory_id)
        )
        return ret.content.decode('utf-8')

    def factory_destroy(self, factory_id):
        ret = self.command(
            'mediaFileFind.cgi?action=factory.destroy&object={0}'
            .format(factory_id)
        )
        return ret.content.decode('utf-8')

    def media_file_find_start(self, factory_id,
                              start_time, end_time, channel=0,
                              directories=(), types=(), flags=(),
                              events=(), stream=None):
        """
        https://s3.amazonaws.com/amcrest-files/Amcrest+HTTP+API+3.2017.pdf

        factory_id : returned by factory_create()

        dir : in which directories you want to find the file. It is an array.
                The index starts from 1. The range of dir is {"/mnt/dvr/sda0",
                "/mnt/dvr/sda1"}. This condition can be omitted. If omitted,
                find files in all the directories.

        type : which types of the file you want to find. It is an array. The
                index starts from 0. The range of type is {"dav", "jpg", "mp4"}
                If omitted, find files with all the types.

        flag : which flags of the file you want to find. It is an array. The
                index starts from 0. The range of flag is {"Timing", "Manual",
                "Marker", "Event", "Mosaic", "Cutout"}. If omitted, find files
                with all the flags.

        event : by which event the record file is triggered. It is an array.
                The index starts from 0. The range of event is {"AlarmLocal",
                "VideoMotion", "VideoLoss", "VideoBlind", "Traffic*"}. This
                condition can be omitted. If omitted, find files of all the
                events. stream : which video stream type you want to find.
                The range of stream is {"Main", "Extra1", "Extra2", "Extra3"}.
                If omitted, find files with all the stream types.
        """

        c_dirs = ''.join(['&condition.Dirs[{0}]={1}'.format(k, v)
                          for k, v in enumerate(directories)])

        c_types = ''.join(['&condition.Types[{0}]={1}'.format(k, v)
                           for k, v in enumerate(types)])

        c_flag = ''.join(['&condition.Flag[{0}]={1}'.format(k, v)
                          for k, v in enumerate(flags)])

        c_events = ''.join(['&condition.Events[{0}]={1}'.format(k, v)
                            for k, v in enumerate(events)])

        c_vs = ('&condition.VideoStream={0}'.format(stream) if stream else '')

        ret = self.command(
            'mediaFileFind.cgi?action=findFile&object={0}&condition.Channel'
            '={1}&condition.StartTime={2}&condition.EndTime={3}{4}{5}{6}{7}{8}'
            .format(factory_id, channel, start_time, end_time, c_dirs, c_types,
                    c_flag, c_events, c_vs)
        )
        return ret.content.decode('utf-8')

    def media_file_find_next(self, factory_id, count=100):
        ret = self.command(
            'mediaFileFind.cgi?action=findNextFile&object={0}&count={1}'
            .format(factory_id, count)
        )

        return ret.content.decode('utf-8')

    def find_files(self, start_time, end_time, channel=0,
                   directories=(), types=(), flags=(), events=(), stream=None):
        """
        https://s3.amazonaws.com/amcrest-files/Amcrest+HTTP+API+3.2017.pdf

        dir : in which directories you want to find the file. It is an array.
                The index starts from 1. The range of dir is {"/mnt/dvr/sda0",
                "/mnt/dvr/sda1"}. This condition can be omitted. If omitted,
                find files in all the directories.

        type : which types of the file you want to find. It is an array. The
                index starts from 0. The range of type is {"dav", "jpg", "mp4"}
                If omitted, find files with all the types.

        flag : which flags of the file you want to find. It is an array. The
                index starts from 0. The range of flag is {"Timing", "Manual",
                "Marker", "Event", "Mosaic", "Cutout"}. If omitted, find files
                with all the flags.

        event : by which event the record file is triggered. It is an array.
                The index starts from 0. The range of event is {"AlarmLocal",
                "VideoMotion", "VideoLoss", "VideoBlind", "Traffic*"}. This
                condition can be omitted. If omitted, find files of all the
                events. stream : which video stream type you want to find.
                The range of stream is {"Main", "Extra1", "Extra2", "Extra3"}.
                If omitted, find files with all the stream types.
        """
        factory_id = self.factory_create().strip().split('=')[1]
        _LOGGER.debug("%s findFile for factory_id=%s", self, factory_id)

        search = self.media_file_find_start(
            factory_id=factory_id,
            start_time=start_time,
            end_time=end_time,
            channel=channel,
            directories=directories,
            types=types,
            flags=flags,
            events=events,
            stream=stream)

        if "ok" in search.lower():
            count = 100

            while count and count > 0:
                _LOGGER.debug("%s findNextFile", self)
                content = self.media_file_find_next(factory_id)

                # The first line is 'found=N'.
                # However, it can be 'Error' if e.g. no more files found
                tag, count = (list(content.split('\r\n', 1)[0]
                                   .split('=')) + [None])[:2]
                _LOGGER.debug("%s returned %s %s", self, tag, count)

                if tag == 'found':
                    count = int(count)
                else:
                    count = None

                yield content

            self.factory_close(factory_id)
            self.factory_destroy(factory_id)
        else:
            _LOGGER.debug("%s returned error: %s", self, search)

    def download_file(self, file_path, timeout=None, stream=False):
        """
        file_path: File location like returned by FilePath from find_files()
                   Example: /mnt/sd/2019-12-31/001/dav/00/00.12.00-00.20.00.mp4
        timeout:   Use default if None
        stream:    If True use streaming download instead of
                   reading content into memory
        """
        ret = self.command(
            'RPC_Loadfile/{0}'.format(file_path),
            timeout_cmd=timeout,
            stream=stream
        )
        return ret.content

    def download_time(self, start_time, end_time, channel=0, stream=0):
        """
        start_time and end_time are formatted as yyyy-mm-dd hh:mm:ss
        '%Y-%m-%d%%20%H:%M:%S'
        """
        ret = self.command(
            'loadfile.cgi?action=startLoad&channel={0}&startTime={1}'
            '&endTime={2}&subtype={3}'
            .format(channel, start_time, end_time, stream)
        )
        return ret.content
# fmt: on



# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# vim:sw=4:ts=4:et


class Log(object):

    @property
    def log_clear_all(self):
        ret = self.command(
            'log.cgi?action=clear'
        )
        return ret.content.decode('utf-8')

    def log_show(self, start_time, end_time):
        ret = self.command(
            'Log.backup?action=All&condition.StartTime='
            '{0}&condition.EndTime={1}'.format(start_time, end_time)
        )
        return ret.content.decode('utf-8')

    def log_find_start(self, start_time, end_time):
        ret = self.command(
            'log.cgi?action=startFind'
            '&condition.StartTime={0}&condition.EndTime={1}'
            .format(
                start_time.strftime('%Y-%m-%d %H:%M:%S'),
                end_time.strftime('%Y-%m-%d %H:%M:%S')))

        return ret.content.decode('utf-8')

    def log_find_next(self, token, count=100):
        ret = self.command('log.cgi?action=doFind&token={0}&count={1}'
                           .format(token, count))
        return ret.content.decode('utf-8')

    def log_find_stop(self, token):
        ret = self.command('log.cgi?action=stopFind&token={0}'
                           .format(token))
        return ret.content.decode('utf-8')

    def log_find(self, start_time, end_time):
        token = self.log_find_start(start_time, end_time).strip().split('=')[1]
        to_query = True

        while to_query:
            content = self.log_find_next(token)
            tag, count = (
                list(content.split('\r\n', 1)[0]
                     .split('=')) + [None])[:2]

            to_query = False

            if (tag == 'found') and (int(count) > 0):
                to_query = True

            yield content

        self.log_find_stop(token)



# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# vim:sw=4:ts=4:et
import logging

from requests import RequestException
from urllib3.exceptions import HTTPError

from .exceptions import CommError

_LOGGER = logging.getLogger(__name__)


def _event_lines(ret):
    line = ""
    for char in ret.iter_content(decode_unicode=True):
        line = line + char
        if line.endswith("\r\n"):
            yield line.strip()
            line = ""


class Event(object):

    def event_handler_config(self, handlername):
        ret = self.command(
            'configManager.cgi?action=getConfig&name={0}'.format(handlername)
        )
        return ret.content.decode('utf-8')

    @property
    def alarm_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=Alarm'
        )
        return ret.content.decode('utf-8')

    @property
    def alarm_out_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=AlarmOut'
        )
        return ret.content.decode('utf-8')

    @property
    def alarm_input_channels(self):
        ret = self.command(
            'alarm.cgi?action=getInSlots'
        )
        return ret.content.decode('utf-8')

    @property
    def alarm_output_channels(self):
        ret = self.command(
            'alarm.cgi?action=getOutSlots'
        )
        return ret.content.decode('utf-8')

    @property
    def alarm_states_input_channels(self):
        ret = self.command(
            'alarm.cgi?action=getInState'
        )
        return ret.content.decode('utf-8')

    @property
    def alarm_states_output_channels(self):
        ret = self.command(
            'alarm.cgi?action=getOutState'
        )
        return ret.content.decode('utf-8')

    @property
    def video_blind_detect_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=BlindDetect'
        )
        return ret.content.decode('utf-8')

    @property
    def video_loss_detect_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=LossDetect'
        )
        return ret.content.decode('utf-8')

    @property
    def event_login_failure(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=LoginFailureAlarm'
        )
        return ret.content.decode('utf-8')

    @property
    def event_storage_not_exist(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=StorageNotExist'
        )
        return ret.content.decode('utf-8')

    @property
    def event_storage_access_failure(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=StorageFailure'
        )
        return ret.content.decode('utf-8')

    @property
    def event_storage_low_space(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=StorageLowSpace'
        )
        return ret.content.decode('utf-8')

    @property
    def event_net_abort(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=NetAbort'
        )
        return ret.content.decode('utf-8')

    @property
    def event_ip_conflict(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=IPConflict'
        )
        return ret.content.decode('utf-8')

    def event_channels_happened(self, eventcode):
        """
        Params:

        VideoMotion: motion detection event
        VideoLoss: video loss detection event
        VideoBlind: video blind detection event
        AlarmLocal: alarm detection event
        StorageNotExist: storage not exist event
        StorageFailure: storage failure event
        StorageLowSpace: storage low space event
        AlarmOutput: alarm output event
        """
        ret = self.command(
            'eventManager.cgi?action=getEventIndexes&code={0}'.format(
                eventcode)
        )
        return ret.content.decode('utf-8')

    @property
    def is_motion_detected(self):
        event = self.event_channels_happened('VideoMotion')
        if 'channels' not in event:
            return False
        return True

    @property
    def is_alarm_triggered(self):
        event = self.event_channels_happened('AlarmLocal')
        return bool('channels' in event)

    @property
    def event_management(self):
        ret = self.command(
            'eventManager.cgi?action=getCaps'
        )
        return ret.content.decode('utf-8')

    def event_stream(self, eventcodes, retries=None, timeout_cmd=None):
        """
        Return a stream of event info lines.

        eventcodes: One or more event codes separated by commas with no spaces

        VideoMotion: motion detection event
        VideoLoss: video loss detection event
        VideoBlind: video blind detection event
        AlarmLocal: alarm detection event
        StorageNotExist: storage not exist event
        StorageFailure: storage failure event
        StorageLowSpace: storage low space event
        AlarmOutput: alarm output event
        """
        urllib3_logger = logging.getLogger("urllib3.connectionpool")
        if not any(isinstance(x, NoHeaderErrorFilter) for x in urllib3_logger.filters):
            urllib3_logger.addFilter(NoHeaderErrorFilter())

        # If timeout is not specified, then use default, but remove read timeout since
        # there's no telling when, if ever, an event will come.
        if timeout_cmd is None:
            try:
                timeout_cmd = (self._timeout_default[0], None)
            except TypeError:
                timeout_cmd = (self._timeout_default, None)

        ret = self.command(
            "eventManager.cgi?action=attach&codes=[{0}]".format(eventcodes),
            retries=retries,
            timeout_cmd=timeout_cmd,
            stream=True,
        )
        if ret.encoding is None:
            ret.encoding = "utf-8"

        try:
            for line in _event_lines(ret):
                if line.lower().startswith("content-length:"):
                    chunk_size = int(line.split(":")[1])
                    yield next(
                        ret.iter_content(chunk_size=chunk_size, decode_unicode=True)
                    )
        except (RequestException, HTTPError) as error:
            _LOGGER.debug("%s Error during event streaming: %r", self, error)
            raise CommError(error)
        finally:
            ret.close()


class NoHeaderErrorFilter(logging.Filter):
    """
    Filter out urllib3 Header Parsing Errors due to a urllib3 bug.

    See https://github.com/urllib3/urllib3/issues/800
    """

    def filter(self, record):
        """Filter out Header Parsing Errors."""
        return "Failed to parse headers" not in record.getMessage()
